import numpy as np

def stellar_density(P, jeff):

   '''
   P = planet orbital period in days
   jeff = ratio of planet semi-major axis to stellar radius
   '''

   G = 6.67e-11 #[m^3/(kg*s^2)]

   print('The density of your star is {} g/cm^3. Sweet.'.format(np.round(((3*np.pi/G) * (1/(P*60*60*24)**2) * (jeff)**3)/1000,2)))


kepler_offset = 2454833.0
tess_offset = 2457000.0

def bjd2bkjd(t):
    '''
    Convert a
    BJD (Barycentric Julian Date)
    into a
    BKJD (Barycentric Kepler Julian Date)
    via the relationship
    BKJD = BJD - 2454833.0
    '''
    return t - kepler_offset


def bkjd2bjd(t):
    '''
    Convert a
    BKJD (Barycentric Kepler Julian Date)
    into a
    BJD (Barycentric Julian Date)
    via the relationship
    BKJD = BJD - 2454833.0
    '''
    return t + kepler_offset

def bjd2btjd(t):
    '''
    Convert a
    BJD (Barycentric Julian Date)
    into a
    BTJD (Barycentric TESS Julian Date)
    via the relationship
    BTJD = BJD - 2457000.0
    '''
    return t - tess_offset


def btjd2bjd(t):
    '''
    Convert a
    BTJD (Barycentric TESS Julian Date)
    into a
    BJD (Barycentric Julian Date)
    via the relationship
    BTJD = BJD - 2457000.0
    '''
    return t + tess_offset



import numpy as np, matplotlib.pyplot as plt
from astropy.modeling.core import Fittable1DModel, custom_model
from astropy.modeling.parameters import Parameter
from astropy.units import Quantity

from .modeling import BATMAN


@custom_model
def TrapezoidTransit(t, delta=0.01, P=1, t0=0, T=0.1, tau=0.01, baseline=1.0):

    """
    One dimensional Trapezoid Transit model,
    using the symbols defined for a circular
    transit approximation in Winn (2010).

    This is a fittable astropy model.

    Parameters
    ----------
    delta : float
        The depth of the transit, as a fraction of the out-of-transit flux.
    P : float
        The perio of the planet, in days.
    t0 : float
        Mid-transit time of the transit, in days.
    T : float
        The duration of the transit (from mid-ingress to mid-egress), in days.
    tau : float
        The duration of ingress/egress, in days.

    See Also
    --------
    astropy.modeling.models.Trapezoid1D

    Examples
    --------
    .. plot::
        :include-source:

        import numpy as np
        import matplotlib.pyplot as plt

        from henrietta.fitting import TrapezoidTransit

        plt.figure()
        model = TrapezoidTransit()
        t = np.arange(-5, 5, .01)

        for x in np.linspace(0.02, 0.2, 4):
            model.delta = x
            model.T = x
            plt.plot(t, model(t), lw=2)

        plt.show()
    """

    # calculate a phase-folded time (still in units of days)
    x = (t-t0 + 0.5*P) % P - 0.5*P

    # Compute the four points where the trapezoid changes slope
    # x1 <= x2 <= x3 <= x4

    if tau > T:
        x1 = -tau
        x2 = 0
        x3 = 0
        x4 = tau
    else:
        x1 = - (T+tau)/2.0
        x2 = - (T-tau)/2.0
        x3 = (T-tau)/2.0
        x4 = (T+tau)/2.0

    # Compute model values in pieces between the change points
    range_a = np.logical_and(x >= x1, x < x2)
    range_b = np.logical_and(x >= x2, x < x3)
    range_c = np.logical_and(x >= x3, x < x4)

    slope = delta/tau
    val_a = 1 - slope * (x - x1)
    val_b = 1 - delta
    val_c = 1 - slope * (x4 - x)
    result = np.select([range_a, range_b, range_c], [val_a, val_b, val_c], default=1)*baseline
    return result

BatmanTransit = custom_model(BATMAN)



from ..modeling import *
from ..statistics import *
from ..goodnesses import *

def test_batman():
    '''
    This function tests the batman transit light curve model.
    '''
    example_transit_model()

def test_residuals(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.5, **kw):
    for e in [True, False]:
        lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0, **kw)
        plot_with_transit_model(lc, period=period, t0=t0, show_errors=e, **kw)

        folded = lc.fold(period=period, phase=t0/period)
        plot_with_transit_model(folded, period=period, t0=t0, show_errors=e, **kw)

    return lc, folded


def test_goodness(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.5, **kw):

    lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0,  **kw)
    folded = lc.fold(period=period, phase=t0/period)
    return plot_with_transit_model(folded, period=period, t0=t0, goodness=chisq, **kw)



from ..fitting import *

def test_trapezoid():
    t = np.linspace(-1.5, 1.5, 10000)

    for tau in np.arange(4)*0.1:
        for T in np.arange(4)*0.1:
            model =  TrapezoidTransit(T=T, tau=tau)
            plt.plot(t, model(t), label=repr(model))
    plt.legend()
    plt.show()


def test_(N=1e6, duration=10, cadence=1.0/60.0/24.0, period=3.14, t0=0.5, **kw):
    lc = simulate_transit_data(N=N, duration=duration, cadence=cadence, period=period, t0=t0, **kw)
    plot_with_transit_model(lc, period=period, t0=t0, show_errors=e, **kw)

    return lc, folded
